@@ -362,8 +362,8 @@ add_underline (PangoRenderer    *renderer,
   int underline_thickness = pango_font_metrics_get_underline_thickness (metrics);
   int underline_position = pango_font_metrics_get_underline_position (metrics);
 
-  new_rect.x = base_x + logical_rect->x;
-  new_rect.width = logical_rect->width;
+  new_rect.x = base_x + ink_rect->x;
+  new_rect.width = ink_rect->width;
   new_rect.height = underline_thickness;
   new_rect.y = base_y;
 
@@ -425,8 +425,8 @@ add_overline (PangoRenderer    *renderer,
   int underline_thickness = pango_font_metrics_get_underline_thickness (metrics);
   int ascent = pango_font_metrics_get_ascent (metrics);
 
-  new_rect.x = base_x + logical_rect->x;
-  new_rect.width = logical_rect->width;
+  new_rect.x = base_x + ink_rect->x;
+  new_rect.width = ink_rect->width;
   new_rect.height = underline_thickness;
   new_rect.y = base_y;
 
@@ -480,8 +480,8 @@ add_strikethrough (PangoRenderer    *renderer,
   int strikethrough_thickness = pango_font_metrics_get_strikethrough_thickness (metrics);
   int strikethrough_position = pango_font_metrics_get_strikethrough_position (metrics);
 
-  new_rect.x = base_x + logical_rect->x;
-  new_rect.width = logical_rect->width;
+  new_rect.x = base_x + ink_rect->x;
+  new_rect.width = ink_rect->width;
   new_rect.y = (base_y - strikethrough_position) * num_glyphs;
   new_rect.height = strikethrough_thickness * num_glyphs;
 
@@ -502,14 +502,10 @@ add_strikethrough (PangoRenderer    *renderer,
 
 static void
 get_item_properties (PangoItem       *item,
-                     gint            *rise,
                      PangoAttrShape **shape_attr)
 {
   GSList *l;
 
-  if (rise)
-    *rise = 0;
-
   if (shape_attr)
     *shape_attr = NULL;
 
@@ -524,11 +520,6 @@ get_item_properties (PangoItem       *item,
             *shape_attr = (PangoAttrShape *)attr;
           break;
 
-        case PANGO_ATTR_RISE:
-          if (rise)
-            *rise = ((PangoAttrInt *)attr)->value;
-          break;
-
         default:
           break;
         }
@@ -616,18 +607,18 @@ pango_renderer_draw_layout_line (PangoRenderer   *renderer,
   for (l = line->runs; l; l = l->next)
     {
       PangoFontMetrics *metrics;
-      gint rise;
       PangoLayoutRun *run = l->data;
       PangoAttrShape *shape_attr;
       PangoRectangle ink_rect, *ink = NULL;
       PangoRectangle logical_rect, *logical = NULL;
+      int y_off;
 
       if (run->item->analysis.flags & PANGO_ANALYSIS_FLAG_CENTERED_BASELINE)
         logical = &logical_rect;
 
       pango_renderer_prepare_run (renderer, run);
 
-      get_item_properties (run->item, &rise, &shape_attr);
+      get_item_properties (run->item, &shape_attr);
 
       if (shape_attr)
         {
@@ -660,6 +651,9 @@ pango_renderer_draw_layout_line (PangoRenderer   *renderer,
 
       state.logical_rect_end = x + x_off + glyph_string_width;
 
+      x_off += run->start_x_offset;
+      y_off = run->y_offset;
+
       if (run->item->analysis.flags & PANGO_ANALYSIS_FLAG_CENTERED_BASELINE)
         {
           gboolean is_hinted = ((logical_rect.y | logical_rect.height) & (PANGO_SCALE - 1)) == 0;
@@ -668,7 +662,7 @@ pango_renderer_draw_layout_line (PangoRenderer   *renderer,
           if (is_hinted)
             adjustment = PANGO_UNITS_ROUND (adjustment);
 
-          rise += adjustment;
+          y_off += adjustment;
         }
 
 
@@ -690,14 +684,14 @@ pango_renderer_draw_layout_line (PangoRenderer   *renderer,
 
       if (shape_attr)
         {
-          draw_shaped_glyphs (renderer, run->glyphs, shape_attr, x + x_off, y - rise);
+          draw_shaped_glyphs (renderer, run->glyphs, shape_attr, x + x_off, y - y_off);
         }
       else
         {
           pango_renderer_draw_glyph_item (renderer,
                                           text,
                                           run,
-                                          x + x_off, y - rise);
+                                          x + x_off, y - y_off);
         }
 
       if (renderer->underline != PANGO_UNDERLINE_NONE ||
@@ -709,17 +703,17 @@ pango_renderer_draw_layout_line (PangoRenderer   *renderer,
 
           if (renderer->underline != PANGO_UNDERLINE_NONE)
             add_underline (renderer, &state,metrics,
-                           x + x_off, y - rise,
+                           x + x_off, y - y_off,
                            ink, logical);
 
           if (renderer->priv->overline != PANGO_OVERLINE_NONE)
             add_overline (renderer, &state,metrics,
-                           x + x_off, y - rise,
+                           x + x_off, y - y_off,
                            ink, logical);
 
           if (renderer->strikethrough)
             add_strikethrough (renderer, &state, metrics,
-                               x + x_off, y - rise,
+                               x + x_off, y - y_off,
                                ink, logical, run->glyphs->num_glyphs);
 
           pango_font_metrics_unref (metrics);
@@ -737,6 +731,7 @@ pango_renderer_draw_layout_line (PangoRenderer   *renderer,
         draw_strikethrough (renderer, &state);
 
       x_off += glyph_string_width;
+      x_off += run->end_x_offset;
     }
 
   /* Finish off any remaining underlines
